==> kuklomenos-0.2.2.tar.gz: kuklomenos-0.2.2/gfx.cc <==
/*
 * Kuklomenos
 * Copyright (C) 2008 Martin Bays <mbays@sdf.lonestar.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#include <SDL/SDL.h>
#include <SDL/SDL_gfxPrimitives.h>

#include "coords.h"
#include "gfx.h"
#include "geom.h"
#include "settings.h"

View::View(CartCoord icentre, float izoom, float iangle) :
    centre(icentre), zoom(izoom), angle(iangle) {}

ScreenCoord View::coord(const CartCoord &c) const
{
    RelCartCoord d( (c-centre).rotated(angle) );
    return ScreenCoord(
	    screenGeom.centre.x+(int)(d.dx*zoom),
	    screenGeom.centre.y-(int)(d.dy*zoom));
}

bool View::inView(const CartCoord &c, float in) const
{
    RelCartCoord d = c-centre;
    return ( (zoom*zoom)*(d.dx*d.dx+d.dy*d.dy) <= (screenGeom.rad-in)*(screenGeom.rad-in) );
}

int Line::draw(SDL_Surface* surface, const View& view, View* boundView, bool blank, bool noAA)
{
    ScreenCoord s = view.coord(start);
    ScreenCoord e = view.coord(end);

    // XXX: HACK! SDL_gfx buggily draws a slightly different line if alpha is
    // 0xff, so we give it 0xfe instead. The incredibly sharp-eyed might be
    // able to see trails as a result...
    Uint32 drawColour = colour;
    if (blank)
	drawColour = (colour & 0x000000ff == 0x000000ff) ?
	    0x000000ff :
	    0x000000fe;

    if (!boundView || (boundView->inView(start) && boundView->inView(end)))
	return ( (!noAA && settings.useAA) ? aalineColor : lineColor )
	    (surface, s.x, s.y, e.x, e.y, drawColour);
    else
	return 0;
}

int Circle::draw(SDL_Surface* surface, const View& view, View* boundView, bool blank, bool noAA)
{
    ScreenCoord c = view.coord(centre);
    if (!boundView || boundView->inView(centre, r*view.zoom))
	return (filled ? filledCircleColor : 
		(!noAA && settings.useAA) ? aacircleColor : circleColor)
	    (surface, c.x, c.y, (int)(r*view.zoom), blank ? 0x000000ff : colour);
    else
	return 0;
}

int Polygon::draw(SDL_Surface* surface, const View& view, View* boundView, bool blank, bool noAA)
{
    for (int i=0; i<n; i++)
    {
	if (boundView && !boundView->inView(points[i]))
	    return 0;
    }

    Sint16 *sx = new Sint16[n];
    Sint16 *sy = new Sint16[n];
    int ret;

    for (int i=0; i<n; i++)
    {
	ScreenCoord s = view.coord(points[i]);
	sx[i] = s.x;
	sy[i] = s.y;
    }
    ret = ( filled ? filledPolygonColor :
	    (!noAA && settings.useAA) ? aapolygonColor : polygonColor )(
		surface, sx, sy, n, blank ? 0x000000ff : colour);

    delete[] sx;
    delete[] sy;
    return ret;
}

int Pixel::draw(SDL_Surface* surface, const View& view, View* boundView, bool blank, bool noAA)
{
    if (boundView && !boundView->inView(point))
	return 0;
    else
    {
	ScreenCoord c = view.coord(point);
	return pixelColor(surface, c.x, c.y, blank ? 0x000000ff : colour);
    }
}

void blankRect(SDL_Surface* surface, int x, int y, int w, int h)
{
    SDL_Rect rect;
    rect.x = x;
    rect.y = y;
    rect.w = w;
    rect.h = h;
    SDL_FillRect(surface, &rect, SDL_MapRGB(surface->format, 0x00, 0x00, 0x00));
}

==> kuklomenos-0.3.tar.gz: kuklomenos-0.3/gfx.cc <==
/*
 * Kuklomenos
 * Copyright (C) 2008 Martin Bays <mbays@sdf.lonestar.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#include <algorithm>
#include <SDL/SDL.h>
#include <SDL_gfxPrimitivesDirty.h>

#include "coords.h"
#include "gfx.h"
#include "geom.h"
#include "settings.h"

View::View(CartCoord icentre, float izoom, float iangle) :
    centre(icentre), zoom(izoom), angle(iangle) {}

ScreenCoord View::coord(const CartCoord &c) const
{
    RelCartCoord d( (c-centre).rotated(angle) );
    return ScreenCoord(
	    screenGeom.centre.x+(int)(d.dx*zoom),
	    screenGeom.centre.y-(int)(d.dy*zoom));
}

bool View::inView(const CartCoord &c, float in) const
{
    RelCartCoord d = c-centre;
    return ( (zoom*zoom)*(d.dx*d.dx+d.dy*d.dy) <= (screenGeom.rad-in)*(screenGeom.rad-in) );
}

int Line::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    ScreenCoord s = view.coord(start);
    ScreenCoord e = view.coord(end);
    const bool useAA = ( (settings.useAA==AA_YES && !noAA) ||
	    settings.useAA==AA_FORCE );

    Uint32 drawColour = colour;

    if (!boundView || (boundView->inView(start) && boundView->inView(end)))
	return ( useAA ? aalineColor : lineColor )
	    (surface, s.x, s.y, e.x, e.y, drawColour);
    else
	return 0;
}

int Circle::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    const bool useAA = ( (settings.useAA==AA_YES && !noAA) ||
	    settings.useAA==AA_FORCE );
    ScreenCoord c = view.coord(centre);
    float screenRad = r*view.zoom;

    if (!boundView || boundView->inView(centre, screenRad))
	return (filled ? filledCircleColor : 
		useAA ? aacircleColor : circleColor)
	    (surface, c.x, c.y, int(screenRad), colour);
    else
	return 0;
}

int Polygon::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    const bool useAA = ( (settings.useAA==AA_YES && !noAA) ||
	    settings.useAA==AA_FORCE );

    for (int i=0; i<n; i++)
    {
	if (boundView && !boundView->inView(points[i]))
	    return 0;
    }

    Sint16 *sx = new Sint16[n];
    Sint16 *sy = new Sint16[n];
    int ret;

    for (int i=0; i<n; i++)
    {
	ScreenCoord s = view.coord(points[i]);
	sx[i] = s.x;
	sy[i] = s.y;
    }

    ret = ( filled ? filledPolygonColor :
	    useAA ? aapolygonColor : polygonColor )(
		surface, sx, sy, n, colour);

    delete[] sx;
    delete[] sy;
    return ret;
}

int Pixel::draw(SDL_Surface* surface, const View& view, View* boundView, bool noAA)
{
    if (boundView && !boundView->inView(point))
	return 0;
    else
    {
	ScreenCoord c = view.coord(point);
	return pixelColor(surface, c.x, c.y, colour);
    }
}
